@@ -80,6 +80,7 @@ var noOpUserReqs = []string{
 	"/file_store",
 	"/universe",
 	"/principals",
+	"/debug",
 }
 
 var apiChan chan *apiTimerInfo
@@ -188,7 +189,7 @@ func main() {
 			os.Exit(1)
 		}
 		errch := make(chan error)
-		go startEventMonitor(serfin.Serfer, errch)
+		go startEventMonitor(config.Config.SerfAddr, errch)
 		err := <-errch
 		if err != nil {
 			logger.Criticalf(err.Error())
@@ -308,12 +309,9 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 			if len(fwded) != 0 {
 				xForwarded = fwded[len(fwded)-1]
 			}
-			logger.Debugf("remote ip candidates: ra: '%s', '%s'", remoteIP, xForwarded)
 			rIP := net.ParseIP(remoteIP)
 			xFIP := net.ParseIP(xForwarded)
-			logger.Debugf("ips now: '%q' '%q'", rIP, xFIP)
-			logger.Debugf("local? '%q' '%q'", rIP.IsLoopback(), xFIP.IsLoopback())
-			if !rIP.IsLoopback() && !xFIP.IsLoopback() {
+			if !rIP.IsLoopback() && !xFIP.IsLoopback() && !config.PprofWhitelisted(rIP) && !config.PprofWhitelisted(xFIP) {
 				logger.Debugf("blocked %s (x-forwarded-for: %s) from accessing /debug/pprof!", rIP.String(), xFIP.String())
 				block = true
 			}
@@ -354,7 +352,6 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	w.Header().Set("X-Chef-Version", config.ChefVersion)
 	apiInfo := fmt.Sprintf("flavor=osc;version:%s;goiardi=%s", config.ChefVersion, config.Version)
 	w.Header().Set("X-Ops-API-Info", apiInfo)
-	w.Header().Set("X-Ops-Server-API-Version", config.ChefApiVersion)
 
 	userID := r.Header.Get("X-OPS-USERID")
 	if rs := r.Header.Get("X-Ops-Request-Source"); rs == "web" {
@@ -380,7 +377,7 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	/* Only perform the authorization check if that's configured. Bomb with
 	 * an error if the check of the headers, timestamps, etc. fails. */
 	/* No clue why /principals doesn't require authorization. Hrmph. */
-	if config.Config.UseAuth && !strings.HasPrefix(r.URL.Path, "/file_store") && !(strings.HasPrefix(r.URL.Path, "/principals") && r.Method == "GET") {
+	if config.Config.UseAuth && !strings.HasPrefix(r.URL.Path, "/file_store") && !strings.HasPrefix(r.URL.Path, "/debug") && !(strings.HasPrefix(r.URL.Path, "/principals") && r.Method == "GET") {
 		herr := authentication.CheckHeader(userID, r)
 		if herr != nil {
 			w.Header().Set("Content-Type", "application/json")
@@ -557,7 +554,7 @@ func handleSignals() {
 					datastore.Dbh.Close()
 				}
 				if config.Config.UseSerf {
-					serfin.Serfer.Close()
+					serfin.CloseAll()
 				}
 				os.Exit(0)
 			} else if sig == syscall.SIGHUP {
@@ -662,54 +659,110 @@ func setLogEventPurgeTicker() {
 	}
 }
 
-func startEventMonitor(sc *serfclient.RPCClient, errch chan<- error) {
-	ch := make(chan map[string]interface{}, 10)
-	sh, err := sc.Stream("*", ch)
+// The serf functionality needs some cleaning up. This is a start on that.
+func startEventMonitor(serfAddr string, errch chan<- error) {
+	// Initial setup of serf. If this bombs go ahead and return so we can
+	// die
+	sc, err := serfin.NewRPCClient(serfAddr)
 	if err != nil {
 		errch <- err
 		return
 	}
 	errch <- nil
 
-	defer sc.Stop(sh)
-	// watch the events and queries
-	for e := range ch {
-		logger.Debugf("Got an event: %v", e)
-		eName, _ := e["Name"]
-		switch eName {
-		case "node_status":
-			jsonPayload := make(map[string]string)
-			err = json.Unmarshal(e["Payload"].([]byte), &jsonPayload)
+	ech := make(chan error)
+	recreateSerfWait := time.Duration(5)
+
+	for {
+		// Make sure the serf client is actually closed before creating
+		// a new one. The very first time this loop is kicked off, of
+		// course, the client will be fine. It's simpler to have the
+		// check up here, though, rather than at the end
+		if sc == nil || sc.IsClosed() {
+			sc, err = serfin.NewRPCClient(serfAddr)
 			if err != nil {
-				logger.Errorf(err.Error())
+				logger.Errorf("Error recreating serf client, waiting %d seconds before recreating: %s", recreateSerfWait, err.Error())
+				time.Sleep(recreateSerfWait * time.Second)
 				continue
+			} else {
+				logger.Errorf("reconnected to serf after being disconnected")
+			}
+		}
+		go runEventMonitor(sc, ech)
+		e := <-ech
+		if e != nil {
+			logger.Errorf("Error from event monitor: %s", e.Error())
+		}
+	}
+}
+
+func runEventMonitor(sc *serfclient.RPCClient, errch chan<- error) {
+	ch := make(chan map[string]interface{}, 10)
+	sh, err := sc.Stream("*", ch)
+	if err != nil {
+		errch <- err
+		return
+	}
+
+	defer sc.Stop(sh)
+	checkClientSec := time.Duration(15)
+
+	// watch the events and queries
+	for {
+		select {
+		case e := <-ch:
+			eNil := e == nil
+			logger.Debugf("Got an event: %v nil? %v", e, eNil)
+			if eNil {
+				if sc.IsClosed() {
+					logger.Debugf("Serf client has been closed, returning from runEventMonitor in hopes of being able to reconnect")
+					err := fmt.Errorf("serf client closed")
+					errch <- err
+					return
+				}
+				continue
+			}
+			eName, _ := e["Name"]
+			switch eName {
+			case "node_status":
+				jsonPayload := make(map[string]string)
+				err = json.Unmarshal(e["Payload"].([]byte), &jsonPayload)
+				if err != nil {
+					logger.Errorf(err.Error())
+					continue
+				}
+				n, _ := node.Get(jsonPayload["node"])
+				if n == nil {
+					logger.Errorf("No node %s", jsonPayload["node"])
+					continue
+				}
+				err = n.UpdateStatus(jsonPayload["status"])
+				if err != nil {
+					logger.Errorf(err.Error())
+					continue
+				}
+				r := map[string]string{"response": "ok"}
+				response, _ := json.Marshal(r)
+				var id uint64
+				switch t := e["ID"].(type) {
+				case int64:
+					id = uint64(t)
+				case uint64:
+					id = t
+				default:
+					logger.Errorf("node_status ID %v type %T not int64 or uint64", e["ID"], e["ID"])
+					continue
+				}
+				sc.Respond(id, response)
+			}
+		case <-time.After(checkClientSec * time.Second):
+			if sc.IsClosed() {
+				clerr := fmt.Errorf("serf client found to be closed, recreating")
+				errch <- clerr
+				return
 			}
-			n, _ := node.Get(jsonPayload["node"])
-			if n == nil {
-				logger.Errorf("No node %s", jsonPayload["node"])
-				continue
-			}
-			err = n.UpdateStatus(jsonPayload["status"])
-			if err != nil {
-				logger.Errorf(err.Error())
-				continue
-			}
-			r := map[string]string{"response": "ok"}
-			response, _ := json.Marshal(r)
-			var id uint64
-			switch t := e["ID"].(type) {
-			case int64:
-				id = uint64(t)
-			case uint64:
-				id = t
-			default:
-				logger.Errorf("node_status ID %v type %T not int64 or uint64", e["ID"], e["ID"])
-				continue
-			}
-			sc.Respond(id, response)
 		}
 	}
-	return
 }
 
 func startNodeMonitor() {
